# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import WalletsService
from .create_wallet import (
    CreateWalletRequest,
    CreateWalletResponse
)
from .create_wallet_address import (
    CreateWalletAddressRequest,
    CreateWalletAddressResponse
)
from .get_wallet import (
    GetWalletRequest,
    GetWalletResponse
)
from .get_wallet_deposit_instructions import (
    GetWalletDepositInstructionsRequest,
    GetWalletDepositInstructionsResponse
)
from .list_wallet_addresses import (
    ListWalletAddressesRequest,
    ListWalletAddressesResponse
)
from .list_wallets import (
    ListWalletsRequest,
    ListWalletsResponse
)

__all__ = [
    "WalletsService",
    "CreateWalletRequest",
    "CreateWalletResponse",
    "CreateWalletAddressRequest",
    "CreateWalletAddressResponse",
    "GetWalletRequest",
    "GetWalletResponse",
    "GetWalletDepositInstructionsRequest",
    "GetWalletDepositInstructionsResponse",
    "ListWalletAddressesRequest",
    "ListWalletAddressesResponse",
    "ListWalletsRequest",
    "ListWalletsResponse"
]


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import PaymentMethodsService
from .get_entity_payment_method import (
    GetEntityPaymentMethodRequest,
    GetEntityPaymentMethodResponse
)
from .list_entity_payment_methods import (
    ListEntityPaymentMethodsRequest,
    ListEntityPaymentMethodsResponse
)

__all__ = [
    "PaymentMethodsService",
    "GetEntityPaymentMethodRequest",
    "GetEntityPaymentMethodResponse",
    "ListEntityPaymentMethodsRequest",
    "ListEntityPaymentMethodsResponse"
]


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import FuturesService
from .cancel_entity_futures_sweep import (
    CancelEntityFuturesSweepRequest,
    CancelEntityFuturesSweepResponse
)
from .schedule_entity_futures_sweep import (
    ScheduleEntityFuturesSweepRequest,
    ScheduleEntityFuturesSweepResponse
)
from .list_entity_futures_sweeps import (
    ListEntityFuturesSweepsRequest,
    ListEntityFuturesSweepsResponse
)
from .set_auto_sweep import (
    SetAutoSweepRequest,
    SetAutoSweepResponse
)
from .get_entity_fcm_balance import (
    GetEntityFcmBalanceRequest,
    GetEntityFcmBalanceResponse
)
from .get_entity_positions import (
    GetEntityPositionsRequest,
    GetEntityPositionsResponse
)

__all__ = [
    "FuturesService",
    "CancelEntityFuturesSweepRequest",
    "CancelEntityFuturesSweepResponse",
    "ScheduleEntityFuturesSweepRequest",
    "ScheduleEntityFuturesSweepResponse",
    "ListEntityFuturesSweepsRequest",
    "ListEntityFuturesSweepsResponse",
    "SetAutoSweepRequest",
    "SetAutoSweepResponse",
    "GetEntityFcmBalanceRequest",
    "GetEntityFcmBalanceResponse",
    "GetEntityPositionsRequest",
    "GetEntityPositionsResponse"
]


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import StakingService
from .create_stake import (
    StakingInputs,
    CreateStakeRequest,
    CreateStakeResponse
)
from .create_unstake import (
    CreateUnstakeRequest,
    CreateUnstakeResponse
)
from .create_portfolio_stake import (
    StakeMetadata,
    CreatePortfolioStakeRequest,
    CreatePortfolioStakeResponse
)
from .create_portfolio_unstake import (
    UnstakeMetadata,
    CreatePortfolioUnstakeRequest,
    CreatePortfolioUnstakeResponse
)

__all__ = [
    "StakingService",
    "StakingInputs",
    "CreateStakeRequest",
    "CreateStakeResponse",
    "CreateUnstakeRequest",
    "CreateUnstakeResponse",
    "StakeMetadata",
    "CreatePortfolioStakeRequest",
    "CreatePortfolioStakeResponse",
    "UnstakeMetadata",
    "CreatePortfolioUnstakeRequest",
    "CreatePortfolioUnstakeResponse"
]


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import asdict
from ...client import Client
from ...utils import append_query_param, append_pagination_params, to_body_dict
from .create_portfolio_allocations import (
    CreatePortfolioAllocationsRequest,
    CreatePortfolioAllocationsResponse
)
from .create_portfolio_net_allocations import (
    CreatePortfolioNetAllocationsRequest,
    CreatePortfolioNetAllocationsResponse
)
from .get_allocation_by_id import (
    GetAllocationByIdRequest,
    GetAllocationByIdResponse
)
from .get_net_allocations_by_netting_id import (
    GetNetAllocationsByNettingIdRequest,
    GetNetAllocationsByNettingIdResponse
)
from .list_portfolio_allocations import (
    ListPortfolioAllocationsRequest,
    ListPortfolioAllocationsResponse
)


class AllocationsService:
    def __init__(self, client: Client):
        self.client = client

    def create_portfolio_allocations(self, request: CreatePortfolioAllocationsRequest) -> CreatePortfolioAllocationsResponse:
        path = "/allocations"

        body = to_body_dict(request)
        if request.allocation_legs:
            body["allocation_legs"] = [asdict(leg) for leg in request.allocation_legs]

        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreatePortfolioAllocationsResponse(**response.json())

    def create_portfolio_net_allocations(self, request: CreatePortfolioNetAllocationsRequest) -> CreatePortfolioNetAllocationsResponse:
        path = "/allocations/net"

        body = to_body_dict(request)
        if request.allocation_legs:
            body["allocation_legs"] = [asdict(leg) for leg in request.allocation_legs]

        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreatePortfolioNetAllocationsResponse(**response.json())

    def get_allocation_by_id(self, request: GetAllocationByIdRequest) -> GetAllocationByIdResponse:
        path = f"/portfolios/{request.portfolio_id}/allocations/{request.allocation_id}"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetAllocationByIdResponse(**response.json())

    def get_net_allocations_by_netting_id(self, request: GetNetAllocationsByNettingIdRequest) -> GetNetAllocationsByNettingIdResponse:
        path = f"/portfolios/{request.portfolio_id}/allocations/net/{request.netting_id}"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetNetAllocationsByNettingIdResponse(**response.json())

    def list_portfolio_allocations(self, request: ListPortfolioAllocationsRequest) -> ListPortfolioAllocationsResponse:
        path = f"/portfolios/{request.portfolio_id}/allocations"

        query_params = append_query_param("", 'product_ids', request.product_ids)
        query_params = append_query_param(query_params, 'order_side', request.order_side)

        if request.start_date:
            query_params = append_query_param(query_params, 'start_date', request.start_date.isoformat() + 'Z')
        if request.end_date:
            query_params = append_query_param(query_params, 'end_date', request.end_date.isoformat() + 'Z')

        query_params = append_pagination_params(query_params, request.pagination)

        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListPortfolioAllocationsResponse(**response.json())


import unittest
from dataclasses import dataclass
from prime_sdk.base_response import BaseResponse

# Create a test response class that inherits from BaseResponse
@dataclass
class TestResponse(BaseResponse):
    field1: str = None
    field2: int = None
    nested_data: dict = None

class TestBaseResponse(unittest.TestCase):
    def test_initialization(self):
        # Test basic field initialization
        test_response = TestResponse(
            field1='value1',
            field2=123,
            nested_data={'nested_field': 'nested_value'}
        )

        self.assertEqual(test_response.field1, 'value1')
        self.assertEqual(test_response.field2, 123)
        self.assertEqual(test_response.nested_data['nested_field'], 'nested_value')

    def test_str_representation(self):
        test_response = TestResponse(field1='value1', field2=123)
        str_repr = str(test_response)
        # Should be JSON representation
        self.assertIn('"field1": "value1"', str_repr)
        self.assertIn('"field2": 123', str_repr)

    def test_empty_initialization(self):
        # Test that BaseResponse can be initialized with default values
        test_response = TestResponse()
        self.assertIsNone(test_response.field1)
        self.assertIsNone(test_response.field2)
        self.assertIsNone(test_response.nested_data)

if __name__ == '__main__':
    unittest.main() 


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ...client import Client
from ...utils import append_query_param, append_pagination_params
from .get_activity import GetActivityRequest, GetActivityResponse
from .get_entity_activity import GetEntityActivityRequest, GetEntityActivityResponse
from .list_activities import ListActivitiesRequest, ListActivitiesResponse
from .list_entity_activities import ListEntityActivitiesRequest, ListEntityActivitiesResponse


class ActivitiesService:
    def __init__(self, client: Client):
        self.client = client

    def get_activity(self, request: GetActivityRequest) -> GetActivityResponse:
        path = f"/portfolios/{request.portfolio_id}/activities/{request.activity_id}"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetActivityResponse(**response.json())

    def get_entity_activity(self, request: GetEntityActivityRequest) -> GetEntityActivityResponse:
        path = f"/activities/{request.activity_id}"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetEntityActivityResponse(**response.json())

    def list_activities(self, request: ListActivitiesRequest) -> ListActivitiesResponse:
        path = f"/portfolios/{request.portfolio_id}/activities"

        query_params = append_query_param("", 'symbols', request.symbols)
        query_params = append_query_param(query_params, 'categories', request.categories)
        query_params = append_query_param(query_params, 'statuses', request.statuses)

        if request.start_time:
            query_params = append_query_param(
                query_params,
                'start_time',
                request.start_time.isoformat() + 'Z')
        if request.end_time:
            query_params = append_query_param(
                query_params,
                'end_time',
                request.end_time.isoformat() + 'Z')

        query_params = append_pagination_params(query_params, request.pagination)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListActivitiesResponse(**response.json())

    def list_entity_activities(self, request: ListEntityActivitiesRequest) -> ListEntityActivitiesResponse:
        path = f"/entities/{request.entity_id}/activities"

        query_params = append_query_param("", 'activity_level', request.activity_level)
        query_params = append_query_param(query_params, 'symbols', request.symbols)
        query_params = append_query_param(query_params, 'categories', request.categories)
        query_params = append_query_param(query_params, 'statuses', request.statuses)

        if request.start_time:
            query_params = append_query_param(
                query_params,
                'start_time',
                request.start_time.isoformat() + 'Z')
        if request.end_time:
            query_params = append_query_param(
                query_params,
                'end_time',
                request.end_time.isoformat() + 'Z')

        query_params = append_pagination_params(query_params, request.pagination)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListEntityActivitiesResponse(**response.json())


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import ActivitiesService
from .get_activity import (
    GetActivityRequest,
    GetActivityResponse
)
from .get_entity_activity import (
    GetEntityActivityRequest,
    GetEntityActivityResponse
)
from .list_activities import (
    ListActivitiesRequest,
    ListActivitiesResponse
)
from .list_entity_activities import (
    ListEntityActivitiesRequest,
    ListEntityActivitiesResponse
)

__all__ = [
    "ActivitiesService",
    "GetActivityRequest",
    "GetActivityResponse",
    "GetEntityActivityRequest", 
    "GetEntityActivityResponse",
    "ListActivitiesRequest",
    "ListActivitiesResponse",
    "ListEntityActivitiesRequest",
    "ListEntityActivitiesResponse"
]


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import asdict
from ...client import Client
from ...utils import append_query_param, append_pagination_params, to_body_dict
from .accept_quote import (
    AcceptQuoteRequest,
    AcceptQuoteResponse
)
from .cancel_order import (
    CancelOrderRequest,
    CancelOrderResponse
)
from .create_order import (
    CreateOrderRequest,
    CreateOrderResponse
)
from .create_order_preview import (
    CreateOrderPreviewRequest,
    CreateOrderPreviewResponse
)
from .create_quote import (
    CreateQuoteRequest,
    CreateQuoteResponse
)
from .get_order import (
    GetOrderRequest,
    GetOrderResponse
)
from .list_open_orders import (
    ListOpenOrdersRequest,
    ListOpenOrdersResponse
)
from .list_order_fills import (
    ListOrderFillsRequest,
    ListOrderFillsResponse
)
from .list_orders import (
    ListOrdersRequest,
    ListOrdersResponse
)
from .list_portfolio_fills import (
    ListPortfolioFillsRequest,
    ListPortfolioFillsResponse
)


class OrdersService:
    def __init__(self, client: Client):
        self.client = client

    def accept_quote(self, request: AcceptQuoteRequest) -> AcceptQuoteResponse:
        path = f"/portfolios/{request.portfolio_id}/accept_quote"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return AcceptQuoteResponse(**response.json())

    def cancel_order(self, request: CancelOrderRequest) -> CancelOrderResponse:
        path = f"/portfolios/{request.portfolio_id}/orders/{request.order_id}/cancel"
        response = self.client.request("POST", path, allowed_status_codes=request.allowed_status_codes)
        return CancelOrderResponse(**response.json())

    def create_order(self, request: CreateOrderRequest) -> CreateOrderResponse:
        path = f"/portfolios/{request.portfolio_id}/order"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateOrderResponse(**response.json())

    def create_order_preview(self, request: CreateOrderPreviewRequest) -> CreateOrderPreviewResponse:
        path = f"/portfolios/{request.portfolio_id}/order_preview"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateOrderPreviewResponse(**response.json())

    def create_quote(self, request: CreateQuoteRequest) -> CreateQuoteResponse:
        path = f"/portfolios/{request.portfolio_id}/rfq"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateQuoteResponse(**response.json())

    def get_order(self, request: GetOrderRequest) -> GetOrderResponse:
        path = f"/portfolios/{request.portfolio_id}/orders/{request.order_id}"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetOrderResponse(**response.json())

    def list_open_orders(self, request: ListOpenOrdersRequest) -> ListOpenOrdersResponse:
        path = f"/portfolios/{request.portfolio_id}/open_orders"

        query_params = append_query_param("", 'product_ids', request.product_ids)
        query_params = append_query_param(query_params, 'order_type', request.order_type)
        query_params = append_query_param(query_params, 'order_side', request.order_side)

        if request.start_date:
            query_params = append_query_param(
                query_params,
                'start_date',
                request.start_date.isoformat() + 'Z')
        if request.end_date:
            query_params = append_query_param(
                query_params,
                'end_date',
                request.end_date.isoformat() + 'Z')

        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListOpenOrdersResponse(**response.json())

    def list_order_fills(self, request: ListOrderFillsRequest) -> ListOrderFillsResponse:
        path = f"/portfolios/{request.portfolio_id}/orders/{request.order_id}/fills"
        query_params = append_pagination_params("", request.pagination)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListOrderFillsResponse(**response.json())

    def list_orders(self, request: ListOrdersRequest) -> ListOrdersResponse:
        path = f"/portfolios/{request.portfolio_id}/orders"

        query_params = append_query_param("", 'order_statuses', request.order_statuses)
        query_params = append_query_param(query_params, 'product_ids', request.product_ids)
        query_params = append_query_param(query_params, 'order_type', request.order_type)
        query_params = append_query_param(query_params, 'order_side', request.order_side)

        if request.start_date:
            query_params = append_query_param(query_params, 'start_date', request.start_date.isoformat() + 'Z')
        if request.end_date:
            query_params = append_query_param(query_params, 'end_date', request.end_date.isoformat() + 'Z')

        query_params = append_pagination_params(query_params, request.pagination)

        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListOrdersResponse(**response.json())

    def list_portfolio_fills(self, request: ListPortfolioFillsRequest) -> ListPortfolioFillsResponse:
        path = f"/portfolios/{request.portfolio_id}/fills"

        query_params = append_query_param("", 'start_date', request.start_date)

        if request.end_date:
            query_params = append_query_param(query_params, 'end_date', request.end_date)

        query_params = append_pagination_params(query_params, request.pagination)

        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListPortfolioFillsResponse(**response.json())


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ...client import Client
from .cancel_entity_futures_sweep import (
    CancelEntityFuturesSweepRequest,
    CancelEntityFuturesSweepResponse
)
from .schedule_entity_futures_sweep import (
    ScheduleEntityFuturesSweepRequest,
    ScheduleEntityFuturesSweepResponse
)
from .list_entity_futures_sweeps import (
    ListEntityFuturesSweepsRequest,
    ListEntityFuturesSweepsResponse
)
from .set_auto_sweep import (
    SetAutoSweepRequest,
    SetAutoSweepResponse
)
from .get_entity_fcm_balance import (
    GetEntityFcmBalanceRequest,
    GetEntityFcmBalanceResponse
)
from .get_entity_positions import (
    GetEntityPositionsRequest,
    GetEntityPositionsResponse
)
from ...utils import to_body_dict


class FuturesService:
    def __init__(self, client: Client):
        self.client = client

    def cancel_entity_futures_sweep(self, request: CancelEntityFuturesSweepRequest) -> CancelEntityFuturesSweepResponse:
        path = f"/entities/{request.entity_id}/futures_sweep"
        response = self.client.request("DELETE", path, allowed_status_codes=request.allowed_status_codes)
        return CancelEntityFuturesSweepResponse(**response.json())

    def schedule_entity_futures_sweep(self, request: ScheduleEntityFuturesSweepRequest) -> ScheduleEntityFuturesSweepResponse:
        path = f"/entities/{request.entity_id}/futures_sweep"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return ScheduleEntityFuturesSweepResponse(**response.json())

    def list_entity_futures_sweeps(self, request: ListEntityFuturesSweepsRequest) -> ListEntityFuturesSweepsResponse:
        path = f"/entities/{request.entity_id}/futures_sweeps"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return ListEntityFuturesSweepsResponse(**response.json())

    def set_auto_sweep(self, request: SetAutoSweepRequest) -> SetAutoSweepResponse:
        path = f"/entities/{request.entity_id}/auto_sweep"
        body = to_body_dict(request)
        response = self.client.request("PUT", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return SetAutoSweepResponse(**response.json())

    def get_entity_fcm_balance(self, request: GetEntityFcmBalanceRequest) -> GetEntityFcmBalanceResponse:
        path = f"/entities/{request.entity_id}/fcm_balance"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetEntityFcmBalanceResponse(**response.json())

    def get_entity_positions(self, request: GetEntityPositionsRequest) -> GetEntityPositionsResponse:
        path = f"/entities/{request.entity_id}/positions"
        query_params = ""
        if request.product_id:
            query_params = f"product_id={request.product_id}"
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return GetEntityPositionsResponse(**response.json())


from prime_sdk.services.orders import GetOrderResponse
from prime_sdk.model import Order


def test_get_order_response_parsing():
    # Complete mock order data with all required fields
    mock_order_data = {
        "id": "order-123",
        "user_id": "user-abc",
        "portfolio_id": "portfolio-xyz",
        "product_id": "BTC-USD",
        "side": "BUY",
        "client_order_id": "client-order-1",
        "type": "LIMIT",
        "base_quantity": "1.0",
        "quote_value": "60000.00",
        "limit_price": "60000.00",
        "start_time": "2024-01-01T00:00:00Z",
        "expiry_time": "2024-01-01T01:00:00Z",
        "status": "FILLED",
        "time_in_force": "GTC",
        "created_at": "2024-01-01T00:00:00Z",
        "filled_quantity": "1.0",
        "filled_value": "60000.00",
        "average_filled_price": "60000.00",
        "commission": "0.00",
        "exchange_fee": "0.00",
        "historical_pov": "100",
        "stop_price": "0.00",
        "net_average_filled_price": "60000.00",
        "user_context": "some-context",
        "client_product_id": "BTC-USD"
    }

    # Initialize with the actual field values
    response = GetOrderResponse(order=mock_order_data)

    assert isinstance(response.order, Order)
    assert response.order.id == "order-123"
    assert response.order.side == "BUY"
    assert response.order.status == "FILLED"
    assert response.order.product_id == "BTC-USD"



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import OrdersService
from .accept_quote import (
    AcceptQuoteRequest,
    AcceptQuoteResponse
)
from .cancel_order import (
    CancelOrderRequest,
    CancelOrderResponse
)
from .create_order import (
    CreateOrderRequest,
    CreateOrderResponse
)
from .create_order_preview import (
    CreateOrderPreviewRequest,
    CreateOrderPreviewResponse
)
from .create_quote import (
    CreateQuoteRequest,
    CreateQuoteResponse
)
from .get_order import (
    GetOrderRequest,
    GetOrderResponse
)
from .list_open_orders import (
    ListOpenOrdersRequest,
    ListOpenOrdersResponse
)
from .list_order_fills import (
    ListOrderFillsRequest,
    ListOrderFillsResponse
)
from .list_orders import (
    ListOrdersRequest,
    ListOrdersResponse
)
from .list_portfolio_fills import (
    ListPortfolioFillsRequest,
    ListPortfolioFillsResponse
)

__all__ = [
    "OrdersService",
    "AcceptQuoteRequest",
    "AcceptQuoteResponse",
    "CancelOrderRequest",
    "CancelOrderResponse",
    "CreateOrderRequest",
    "CreateOrderResponse",
    "CreateOrderPreviewRequest",
    "CreateOrderPreviewResponse",
    "CreateQuoteRequest",
    "CreateQuoteResponse",
    "GetOrderRequest",
    "GetOrderResponse",
    "ListOpenOrdersRequest",
    "ListOpenOrdersResponse",
    "ListOrderFillsRequest",
    "ListOrderFillsResponse",
    "ListOrdersRequest",
    "ListOrdersResponse",
    "ListPortfolioFillsRequest",
    "ListPortfolioFillsResponse"
]


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import UsersService
from .list_entity_users import (
    ListEntityUsersRequest,
    ListEntityUsersResponse
)
from .list_portfolio_users import (
    ListPortfolioUsersRequest,
    ListPortfolioUsersResponse
)

__all__ = [
    "UsersService",
    "ListEntityUsersRequest",
    "ListEntityUsersResponse",
    "ListPortfolioUsersRequest",
    "ListPortfolioUsersResponse"
]


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ...client import Client
from ...utils import append_pagination_params
from .list_entity_users import (
    ListEntityUsersRequest,
    ListEntityUsersResponse
)
from .list_portfolio_users import (
    ListPortfolioUsersRequest,
    ListPortfolioUsersResponse
)


class UsersService:
    def __init__(self, client: Client):
        self.client = client

    def list_entity_users(self, request: ListEntityUsersRequest) -> ListEntityUsersResponse:
        path = f"/entities/{request.entity_id}/users"
        query_params = append_pagination_params("", request.pagination)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListEntityUsersResponse(**response.json())

    def list_portfolio_users(self, request: ListPortfolioUsersRequest) -> ListPortfolioUsersResponse:
        path = f"/portfolios/{request.portfolio_id}/users"
        query_params = append_pagination_params("", request.pagination)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListPortfolioUsersResponse(**response.json())


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import List, Optional
from warnings import warn
from ...base_response import BaseResponse
from ...enums import SizeType


@dataclass
class AllocationLeg:
    allocation_leg_id: str
    destination_portfolio_id: str
    amount: str
    leg_id: Optional[str] = None

    def __post_init__(self):
        if self.leg_id:
            warn("The 'leg_id' field is deprecated and will be removed in a future version. Use 'allocation_leg_id' instead.", DeprecationWarning)
            self.allocation_leg_id = self.leg_id
        else:
            self.leg_id = self.allocation_leg_id


@dataclass
class CreatePortfolioAllocationsRequest:
    allocation_id: str
    source_portfolio_id: str
    product_id: str
    order_ids: List[str]
    allocation_legs: List[AllocationLeg]
    size_type: SizeType
    remainder_destination_portfolio_id: str
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class CreatePortfolioAllocationsResponse(BaseResponse):
    success: bool = None
    allocation_id: str = None
    failure_reason: str = None



# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import List, Optional
from ...base_response import BaseResponse


@dataclass
class Rpc:
    skip_broadcast: Optional[bool] = None
    url: Optional[str] = None


@dataclass
class EvmParams:
    disable_dynamic_gas: Optional[bool] = None
    replaced_transaction_id: Optional[str] = None
    chain_id: Optional[str] = None


@dataclass
class CreateOnchainTransactionRequest:
    portfolio_id: str
    wallet_id: str
    raw_unsigned_txn: str
    rpc: Optional[Rpc] = None
    evm_params: Optional[EvmParams] = None
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class CreateOnchainTransactionResponse(BaseResponse):
    transaction_id: str = None


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ...client import Client
from .get_portfolio_commission import GetPortfolioCommissionRequest, GetPortfolioCommissionResponse


class CommissionService:
    def __init__(self, client: Client):
        self.client = client

    def get_portfolio_commission(self, request: GetPortfolioCommissionRequest) -> GetPortfolioCommissionResponse:
        path = f"/portfolios/{request.portfolio_id}/commission"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetPortfolioCommissionResponse(**response.json())


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import FinancingService
from .create_new_locate import (
    CreateNewLocateRequest,
    CreateNewLocateResponse
)
from .list_existing_locates import (
    ListExistingLocatesRequest,
    ListExistingLocatesResponse
)
from .get_entity_locate_availabilities import (
    GetEntityLocateAvailabilitiesRequest,
    GetEntityLocateAvailabilitiesResponse
)
from .get_portfolio_buying_power import (
    GetBuyingPowerRequest,
    GetBuyingPowerResponse
)
from .get_portfolio_withdrawal_power import (
    GetPortfolioWithdrawalPowerRequest,
    GetPortfolioWithdrawalPowerResponse
)
from .get_margin_information import (
    GetMarginInformationRequest,
    GetMarginInformationResponse
)
from .list_margin_call_summaries import (
    ListMarginCallSummariesRequest,
    ListMarginCallSummariesResponse
)
from .list_margin_conversions import (
    ListMarginConversionsRequest,
    ListMarginConversionsResponse
)
from .get_portfolio_credit_information import (
    GetPortfolioCreditInformationRequest,
    GetPortfolioCreditInformationResponse
)
from .get_trade_finance_tiered_pricing_fees import (
    GetTradeFinanceTieredPricingFeesRequest,
    GetTradeFinanceTieredPricingFeesResponse
)
from .list_interest_accruals import (
    ListInterestAccrualsRequest,
    ListInterestAccrualsResponse
)
from .list_interest_accruals_for_portfolio import (
    ListInterestAccrualsForPortfolioRequest,
    ListInterestAccrualsForPortfolioResponse
)

__all__ = [
    "FinancingService",
    "CreateNewLocateRequest",
    "CreateNewLocateResponse",
    "ListExistingLocatesRequest",
    "ListExistingLocatesResponse",
    "GetEntityLocateAvailabilitiesRequest",
    "GetEntityLocateAvailabilitiesResponse",
    "GetBuyingPowerRequest",
    "GetBuyingPowerResponse",
    "GetPortfolioWithdrawalPowerRequest",
    "GetPortfolioWithdrawalPowerResponse",
    "GetMarginInformationRequest",
    "GetMarginInformationResponse",
    "ListMarginCallSummariesRequest",
    "ListMarginCallSummariesResponse",
    "ListMarginConversionsRequest",
    "ListMarginConversionsResponse",
    "GetPortfolioCreditInformationRequest",
    "GetPortfolioCreditInformationResponse",
    "GetTradeFinanceTieredPricingFeesRequest",
    "GetTradeFinanceTieredPricingFeesResponse",
    "ListInterestAccrualsRequest",
    "ListInterestAccrualsResponse",
    "ListInterestAccrualsForPortfolioRequest",
    "ListInterestAccrualsForPortfolioResponse"
]


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import OnchainAddressBookService
from .create_onchain_address_book_entry import (
    CreateOnchainAddressBookEntryRequest,
    CreateOnchainAddressBookEntryResponse
)
from .delete_onchain_address_group import (
    DeleteOnchainAddressGroupRequest,
    DeleteOnchainAddressGroupResponse
)
from .list_onchain_address_groups import (
    ListOnchainAddressGroupsRequest,
    ListOnchainAddressGroupsResponse
)
from .update_onchain_address_book import (
    UpdateOnchainAddressBookRequest,
    UpdateOnchainAddressBookResponse
)

__all__ = [
    "OnchainAddressBookService",
    "CreateOnchainAddressBookEntryRequest",
    "CreateOnchainAddressBookEntryResponse",
    "DeleteOnchainAddressGroupRequest",
    "DeleteOnchainAddressGroupResponse",
    "ListOnchainAddressGroupsRequest",
    "ListOnchainAddressGroupsResponse",
    "UpdateOnchainAddressBookRequest",
    "UpdateOnchainAddressBookResponse"
]


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import BalancesService
from .get_wallet_balance import (
    GetWalletBalanceRequest,
    GetWalletBalanceResponse
)
from .list_entity_balances import (
    ListEntityBalancesRequest,
    ListEntityBalancesResponse
)
from .list_web3_wallet_balances import (
    ListWeb3WalletBalancesRequest,
    ListWeb3WalletBalancesResponse
)
from .list_portfolio_balances import (
    ListPortfolioBalancesRequest,
    ListPortfolioBalancesResponse
)

__all__ = [
    "BalancesService",
    "GetWalletBalanceRequest",
    "GetWalletBalanceResponse",
    "ListEntityBalancesRequest",
    "ListEntityBalancesResponse", 
    "ListWeb3WalletBalancesRequest",
    "ListWeb3WalletBalancesResponse",
    "ListPortfolioBalancesRequest",
    "ListPortfolioBalancesResponse"
]


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from prime_sdk.client import Client
from prime_sdk.utils import append_pagination_params
from .get_entity_payment_method import (
    GetEntityPaymentMethodRequest,
    GetEntityPaymentMethodResponse
)
from .list_entity_payment_methods import (
    ListEntityPaymentMethodsRequest,
    ListEntityPaymentMethodsResponse
)


class PaymentMethodsService:
    def __init__(self, client: Client):
        self.client = client

    def get_entity_payment_method(self, request: GetEntityPaymentMethodRequest) -> GetEntityPaymentMethodResponse:
        path = f"/entities/{request.entity_id}/payment-methods/{request.payment_method_id}"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetEntityPaymentMethodResponse(**response.json())

    def list_entity_payment_methods(self, request: ListEntityPaymentMethodsRequest) -> ListEntityPaymentMethodsResponse:
        path = f"/entities/{request.entity_id}/payment-methods"
        query_params = append_pagination_params("", request.pagination)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListEntityPaymentMethodsResponse(**response.json())


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import asdict
from ...client import Client
from ...utils import append_query_param, to_body_dict
from .create_new_locate import (
    CreateNewLocateRequest,
    CreateNewLocateResponse
)
from .list_existing_locates import (
    ListExistingLocatesRequest,
    ListExistingLocatesResponse
)
from .get_entity_locate_availabilities import (
    GetEntityLocateAvailabilitiesRequest,
    GetEntityLocateAvailabilitiesResponse
)
from .get_portfolio_buying_power import (
    GetBuyingPowerRequest,
    GetBuyingPowerResponse
)
from .get_portfolio_withdrawal_power import (
    GetPortfolioWithdrawalPowerRequest,
    GetPortfolioWithdrawalPowerResponse
)
from .get_margin_information import (
    GetMarginInformationRequest,
    GetMarginInformationResponse
)
from .list_margin_call_summaries import (
    ListMarginCallSummariesRequest,
    ListMarginCallSummariesResponse
)
from .list_margin_conversions import (
    ListMarginConversionsRequest,
    ListMarginConversionsResponse
)
from .get_portfolio_credit_information import (
    GetPortfolioCreditInformationRequest,
    GetPortfolioCreditInformationResponse
)
from .get_trade_finance_tiered_pricing_fees import (
    GetTradeFinanceTieredPricingFeesRequest,
    GetTradeFinanceTieredPricingFeesResponse
)
from .list_interest_accruals import (
    ListInterestAccrualsRequest,
    ListInterestAccrualsResponse
)
from .list_interest_accruals_for_portfolio import (
    ListInterestAccrualsForPortfolioRequest,
    ListInterestAccrualsForPortfolioResponse
)


class FinancingService:
    def __init__(self, client: Client):
        self.client = client

    # Locates
    def create_new_locate(self, request: CreateNewLocateRequest) -> CreateNewLocateResponse:
        path = f"/portfolios/{request.portfolio_id}/locates"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateNewLocateResponse(**response.json())

    def list_existing_locates(self, request: ListExistingLocatesRequest) -> ListExistingLocatesResponse:
        path = f"/portfolios/{request.portfolio_id}/locates"
        query_params = append_query_param("", "locate_ids", request.locate_ids)
        query_params = append_query_param(query_params, "locate_date", request.locate_date)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListExistingLocatesResponse(**response.json())

    def get_entity_locate_availabilities(self, request: GetEntityLocateAvailabilitiesRequest) -> GetEntityLocateAvailabilitiesResponse:
        path = f"/entities/{request.entity_id}/locates_availability"
        query_params = append_query_param("", "locate_date", request.locate_date)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return GetEntityLocateAvailabilitiesResponse(**response.json())

    # Power
    def get_portfolio_buying_power(self, request: GetBuyingPowerRequest) -> GetBuyingPowerResponse:
        path = f"/portfolios/{request.portfolio_id}/buying_power"
        query_params = append_query_param("", "base_currency", request.base_currency)
        query_params = append_query_param(query_params, "quote_currency", request.quote_currency)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return GetBuyingPowerResponse(**response.json())

    def get_portfolio_withdrawal_power(self, request: GetPortfolioWithdrawalPowerRequest) -> GetPortfolioWithdrawalPowerResponse:
        path = f"/portfolios/{request.portfolio_id}/withdrawal_power"
        query_params = append_query_param("", "symbol", request.symbol)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return GetPortfolioWithdrawalPowerResponse(**response.json())

    # Margin
    def get_margin_information(self, request: GetMarginInformationRequest) -> GetMarginInformationResponse:
        path = f"/entities/{request.entity_id}/margin"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetMarginInformationResponse(**response.json())

    def list_margin_call_summaries(self, request: ListMarginCallSummariesRequest) -> ListMarginCallSummariesResponse:
        path = f"/entities/{request.entity_id}/margin_summaries"
        query_params = append_query_param("", "start_date", request.start_date)
        query_params = append_query_param(query_params, "end_date", request.end_date)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListMarginCallSummariesResponse(**response.json())

    def list_margin_conversions(self, request: ListMarginConversionsRequest) -> ListMarginConversionsResponse:
        path = f"/portfolios/{request.portfolio_id}/margin_conversions"
        query_params = append_query_param("", "start_date", request.start_date)
        query_params = append_query_param(query_params, "end_date", request.end_date)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListMarginConversionsResponse(**response.json())

    # Credit
    def get_portfolio_credit_information(self, request: GetPortfolioCreditInformationRequest) -> GetPortfolioCreditInformationResponse:
        path = f"/portfolios/{request.portfolio_id}/credit"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetPortfolioCreditInformationResponse(**response.json())

    # Fees
    def get_trade_finance_tiered_pricing_fees(self, request: GetTradeFinanceTieredPricingFeesRequest) -> GetTradeFinanceTieredPricingFeesResponse:
        path = f"/portfolios/{request.portfolio_id}/tf_tiered_fees"
        query_params = append_query_param("", "effective_at", request.effective_at)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return GetTradeFinanceTieredPricingFeesResponse(**response.json())

    # Interest Accruals
    def list_interest_accruals(self, request: ListInterestAccrualsRequest) -> ListInterestAccrualsResponse:
        path = f"/entities/{request.entity_id}/accruals"
        query_params = append_query_param("", "portfolio_id", request.portfolio_id)
        query_params = append_query_param(query_params, "start_date", request.start_date)
        query_params = append_query_param(query_params, "end_date", request.end_date)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListInterestAccrualsResponse(**response.json())

    def list_interest_accruals_for_portfolio(self, request: ListInterestAccrualsForPortfolioRequest) -> ListInterestAccrualsForPortfolioResponse:
        path = f"/portfolios/{request.portfolio_id}/accruals"
        query_params = append_query_param("", "start_date", request.start_date)
        query_params = append_query_param(query_params, "end_date", request.end_date)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListInterestAccrualsForPortfolioResponse(**response.json())


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from prime_sdk.client import Client
from prime_sdk.utils import append_pagination_params, to_body_dict
from .create_wallet import (
    CreateWalletRequest,
    CreateWalletResponse
)
from .create_wallet_address import (
    CreateWalletAddressRequest,
    CreateWalletAddressResponse
)
from .get_wallet import (
    GetWalletRequest,
    GetWalletResponse
)
from .get_wallet_deposit_instructions import (
    GetWalletDepositInstructionsRequest,
    GetWalletDepositInstructionsResponse
)
from .list_wallet_addresses import (
    ListWalletAddressesRequest,
    ListWalletAddressesResponse
)
from .list_wallets import (
    ListWalletsRequest,
    ListWalletsResponse
)


class WalletsService:
    def __init__(self, client: Client):
        self.client = client

    def create_wallet(self, request: CreateWalletRequest) -> CreateWalletResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateWalletResponse(**response.json())

    def create_wallet_address(self, request: CreateWalletAddressRequest) -> CreateWalletAddressResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/addresses"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateWalletAddressResponse(**response.json())

    def get_wallet(self, request: GetWalletRequest) -> GetWalletResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetWalletResponse(**response.json())

    def get_wallet_deposit_instructions(self, request: GetWalletDepositInstructionsRequest) -> GetWalletDepositInstructionsResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/deposit_instructions"
        query_params = f"deposit_type={request.deposit_type.value}"
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return GetWalletDepositInstructionsResponse(**response.json())

    def list_wallet_addresses(self, request: ListWalletAddressesRequest) -> ListWalletAddressesResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/addresses"
        query_params = f"network_id={request.network_id}"
        query_params = append_pagination_params(query_params, request.pagination)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListWalletAddressesResponse(**response.json())

    def list_wallets(self, request: ListWalletsRequest) -> ListWalletsResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets"
        query_params = append_pagination_params("", request.pagination)
        if request.type:
            query_params = f"{query_params}&type={request.type.value}" if query_params else f"type={request.type.value}"
        if request.symbols:
            query_params = f"{query_params}&symbols={request.symbols}" if query_params else f"symbols={request.symbols}"
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListWalletsResponse(**response.json())


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import Optional, List
from ...model import Candle
from ...base_response import BaseResponse
from ...utils import PaginationParams, Pagination


@dataclass
class GetProductCandlesRequest:
    portfolio_id: str
    product_id: str
    granularity: str
    start_time: int
    end_time: int
    pagination: Optional[PaginationParams] = None
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class GetProductCandlesResponse(BaseResponse):
    candles: List[Candle] = None
    pagination: Pagination = None


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from prime_sdk.client import Client
from prime_sdk.utils import append_pagination_params, to_body_dict
from .create_conversion import (
    CreateConversionRequest,
    CreateConversionResponse
)
from .create_onchain_transaction import (
    CreateOnchainTransactionRequest,
    CreateOnchainTransactionResponse
)
from .create_transfer import (
    CreateTransferRequest,
    CreateTransferResponse
)
from .create_withdrawal import (
    CreateWithdrawalRequest,
    CreateWithdrawalResponse
)
from .get_transaction import (
    GetTransactionRequest,
    GetTransactionResponse
)
from .list_portfolio_transactions import (
    ListPortfolioTransactionsRequest,
    ListPortfolioTransactionsResponse
)
from .list_wallet_transactions import (
    ListWalletTransactionsRequest,
    ListWalletTransactionsResponse
)


class TransactionsService:
    def __init__(self, client: Client):
        self.client = client

    def create_conversion(self, request: CreateConversionRequest) -> CreateConversionResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/conversions"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateConversionResponse(**response.json())

    def create_onchain_transaction(self, request: CreateOnchainTransactionRequest) -> CreateOnchainTransactionResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/transactions"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateOnchainTransactionResponse(**response.json())

    def create_transfer(self, request: CreateTransferRequest) -> CreateTransferResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/transfers"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateTransferResponse(**response.json())

    def create_withdrawal(self, request: CreateWithdrawalRequest) -> CreateWithdrawalResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/withdrawals"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateWithdrawalResponse(**response.json())

    def get_transaction(self, request: GetTransactionRequest) -> GetTransactionResponse:
        path = f"/portfolios/{request.portfolio_id}/transactions/{request.transaction_id}"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetTransactionResponse(**response.json())

    def list_portfolio_transactions(self, request: ListPortfolioTransactionsRequest) -> ListPortfolioTransactionsResponse:
        path = f"/portfolios/{request.portfolio_id}/transactions"
        query_params = append_pagination_params("", request.pagination)
        if request.symbols:
            query_params = f"{query_params}&symbols={request.symbols}" if query_params else f"symbols={request.symbols}"
        if request.types:
            query_params = f"{query_params}&types={request.types}" if query_params else f"types={request.types}"
        if request.start:
            query_params = f"{query_params}&start={request.start.isoformat()}" if query_params else f"start={request.start.isoformat()}"
        if request.end:
            query_params = f"{query_params}&end={request.end.isoformat()}" if query_params else f"end={request.end.isoformat()}"
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListPortfolioTransactionsResponse(**response.json())

    def list_wallet_transactions(self, request: ListWalletTransactionsRequest) -> ListWalletTransactionsResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/transactions"
        query_params = append_pagination_params("", request.pagination)
        if request.types:
            query_params = f"{query_params}&types={request.types}" if query_params else f"types={request.types}"
        if request.start:
            query_params = f"{query_params}&start={request.start.isoformat()}" if query_params else f"start={request.start.isoformat()}"
        if request.end:
            query_params = f"{query_params}&end={request.end.isoformat()}" if query_params else f"end={request.end.isoformat()}"
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListWalletTransactionsResponse(**response.json())


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import AddressBookService
from .create_address_book_entry import (
    CreateAddressBookEntryRequest,
    CreateAddressBookEntryResponse
)
from .get_address_book import (
    GetAddressBookRequest,
    GetAddressBookResponse
)

__all__ = [
    "AddressBookService",
    "CreateAddressBookEntryRequest",
    "CreateAddressBookEntryResponse",
    "GetAddressBookRequest",
    "GetAddressBookResponse"
]


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import dataclasses
from dataclasses import dataclass, fields, asdict
from typing import get_type_hints

@dataclass
class BaseResponse:
    def __post_init__(self):
        type_hints = get_type_hints(self.__class__)
        for f in fields(self):
            value = getattr(self, f.name)
            if value is None:
                continue
            expected_type = type_hints.get(f.name)
            if hasattr(expected_type, '__origin__') and expected_type.__origin__ is list:
                inner_type = expected_type.__args__[0]
                if dataclasses.is_dataclass(inner_type) and isinstance(value, list):
                    setattr(self, f.name, [inner_type(**v) if isinstance(v, dict) else v for v in value])
            elif dataclasses.is_dataclass(expected_type) and isinstance(value, dict):
                setattr(self, f.name, expected_type(**value))

    def __str__(self):
        return json.dumps(asdict(self), indent=2)

    def __repr__(self):
        return self.__str__()


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ...client import Client
from ...utils import append_query_param, append_pagination_params, to_body_dict
from .create_address_book_entry import CreateAddressBookEntryRequest, CreateAddressBookEntryResponse
from .get_address_book import GetAddressBookRequest, GetAddressBookResponse


class AddressBookService:
    def __init__(self, client: Client):
        self.client = client

    def create_address_book_entry(self, request: CreateAddressBookEntryRequest) -> CreateAddressBookEntryResponse:
        path = f"/portfolios/{request.portfolio_id}/address_book"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateAddressBookEntryResponse(**response.json())

    def get_address_book(self, request: GetAddressBookRequest) -> GetAddressBookResponse:
        path = f"/portfolios/{request.portfolio_id}/address_book"

        query_params = append_query_param("", 'currency_symbol', request.currency_symbol)
        query_params = append_query_param(query_params, 'search', request.search)
        query_params = append_pagination_params(query_params, request.pagination)

        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return GetAddressBookResponse(**response.json())


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import TransactionsService
from .create_conversion import (
    CreateConversionRequest,
    CreateConversionResponse
)
from .create_onchain_transaction import (
    Rpc,
    EvmParams,
    CreateOnchainTransactionRequest,
    CreateOnchainTransactionResponse
)
from .create_transfer import (
    CreateTransferRequest,
    CreateTransferResponse
)
from .create_withdrawal import (
    PaymentMethod,
    BlockchainAddress,
    CreateWithdrawalRequest,
    CreateWithdrawalResponse
)
from .get_transaction import (
    GetTransactionRequest,
    GetTransactionResponse
)
from .list_portfolio_transactions import (
    ListPortfolioTransactionsRequest,
    ListPortfolioTransactionsResponse
)
from .list_wallet_transactions import (
    ListWalletTransactionsRequest,
    ListWalletTransactionsResponse
)

__all__ = [
    "TransactionsService",
    "CreateConversionRequest",
    "CreateConversionResponse",
    "Rpc",
    "EvmParams",
    "CreateOnchainTransactionRequest",
    "CreateOnchainTransactionResponse",
    "CreateTransferRequest",
    "CreateTransferResponse",
    "PaymentMethod",
    "BlockchainAddress",
    "CreateWithdrawalRequest",
    "CreateWithdrawalResponse",
    "GetTransactionRequest",
    "GetTransactionResponse",
    "ListPortfolioTransactionsRequest",
    "ListPortfolioTransactionsResponse",
    "ListWalletTransactionsRequest",
    "ListWalletTransactionsResponse"
]


# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .service import AllocationsService
from .create_portfolio_allocations import (
    CreatePortfolioAllocationsRequest,
    CreatePortfolioAllocationsResponse,
    AllocationLeg
)
from .create_portfolio_net_allocations import (
    CreatePortfolioNetAllocationsRequest,
    CreatePortfolioNetAllocationsResponse,
    NetAllocationLeg
)
from .get_allocation_by_id import (
    GetAllocationByIdRequest,
    GetAllocationByIdResponse
)
from .get_net_allocations_by_netting_id import (
    GetNetAllocationsByNettingIdRequest,
    GetNetAllocationsByNettingIdResponse
)
from .list_portfolio_allocations import (
    ListPortfolioAllocationsRequest,
    ListPortfolioAllocationsResponse
)

__all__ = [
    "AllocationsService",
    "CreatePortfolioAllocationsRequest",
    "CreatePortfolioAllocationsResponse",
    "AllocationLeg",
    "CreatePortfolioNetAllocationsRequest", 
    "CreatePortfolioNetAllocationsResponse",
    "NetAllocationLeg",
    "GetAllocationByIdRequest",
    "GetAllocationByIdResponse",
    "GetNetAllocationsByNettingIdRequest",
    "GetNetAllocationsByNettingIdResponse",
    "ListPortfolioAllocationsRequest",
    "ListPortfolioAllocationsResponse"
]


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import List, Optional
from ...base_response import BaseResponse
from ...model import Blockchain


@dataclass
class PaymentMethod:
    payment_method_id: str


@dataclass
class Network:
    id: str
    type: str


@dataclass
class BlockchainAddress:
    address: str
    account_identifier: Optional[str] = None
    network: Optional[Network] = None


@dataclass
class Counterparty:
    counterparty_id: str


@dataclass
class CreateWithdrawalRequest:
    portfolio_id: str
    wallet_id: str
    amount: str
    destination_type: str
    idempotency_key: str
    currency_symbol: str
    payment_method: Optional[PaymentMethod] = None
    blockchain_address: Optional[BlockchainAddress] = None
    counterparty: Optional[Counterparty] = None
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class CreateWithdrawalResponse(BaseResponse):
    activity_id: str = None
    approval_url: str = None
    symbol: str = None
    amount: str = None
    fee: str = None
    destination_type: str = None
    source_type: str = None
    blockchain_destination: Blockchain = None
    blockchain_source: Blockchain = None
    transaction_id: str = None


from prime_sdk.services.portfolios import ListPortfoliosResponse
from prime_sdk.model import Portfolio


def test_list_portfolios_response_parsing():
    # Complete mock portfolio data with all required fields
    mock_portfolios_data = [
        {
            "id": "fake-portfolio-id-123",
            "name": "CryptoBalances",
            "entity_id": "fake-entity-id",
            "organization_id": "fake-org-id",
            "entity_name": "Sample Prime Entity"
        }
    ]

    # Initialize with the actual field values
    response = ListPortfoliosResponse(portfolios=mock_portfolios_data)

    assert isinstance(response.portfolios, list)
    assert len(response.portfolios) == 1
    assert isinstance(response.portfolios[0], Portfolio)

    portfolio = response.portfolios[0]
    assert portfolio.id == "fake-portfolio-id-123"
    assert portfolio.name == "CryptoBalances"
    assert portfolio.entity_id == "fake-entity-id"
    assert portfolio.organization_id == "fake-org-id"
    assert portfolio.entity_name == "Sample Prime Entity"



# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import asdict
from ...client import Client
from ...utils import to_body_dict
from .create_onchain_address_book_entry import (
    CreateOnchainAddressBookEntryRequest,
    CreateOnchainAddressBookEntryResponse
)
from .delete_onchain_address_group import (
    DeleteOnchainAddressGroupRequest,
    DeleteOnchainAddressGroupResponse
)
from .list_onchain_address_groups import (
    ListOnchainAddressGroupsRequest,
    ListOnchainAddressGroupsResponse
)
from .update_onchain_address_book import (
    UpdateOnchainAddressBookRequest,
    UpdateOnchainAddressBookResponse
)


class OnchainAddressBookService:
    def __init__(self, client: Client):
        self.client = client

    def create_onchain_address_book_entry(self, request: CreateOnchainAddressBookEntryRequest) -> CreateOnchainAddressBookEntryResponse:
        path = f"/portfolios/{request.portfolio_id}/onchain_address_group"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateOnchainAddressBookEntryResponse(**response.json())

    def delete_onchain_address_group(self, request: DeleteOnchainAddressGroupRequest) -> DeleteOnchainAddressGroupResponse:
        path = f"/portfolios/{request.portfolio_id}/onchain_address_group/{request.address_group_id}"
        response = self.client.request("DELETE", path, allowed_status_codes=request.allowed_status_codes)
        return DeleteOnchainAddressGroupResponse(**response.json())

    def list_onchain_address_groups(self, request: ListOnchainAddressGroupsRequest) -> ListOnchainAddressGroupsResponse:
        path = f"/portfolios/{request.portfolio_id}/onchain_address_groups"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return ListOnchainAddressGroupsResponse(**response.json())

    def update_onchain_address_book(self, request: UpdateOnchainAddressBookRequest) -> UpdateOnchainAddressBookResponse:
        path = f"/portfolios/{request.portfolio_id}/onchain_address_group"
        body = to_body_dict(request)
        response = self.client.request("PUT", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return UpdateOnchainAddressBookResponse(**response.json())


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from prime_sdk.client import Client
from prime_sdk.utils import append_pagination_params
from .list_products import (
    ListProductsRequest,
    ListProductsResponse
)
from .get_product_candles import (
    GetProductCandlesRequest,
    GetProductCandlesResponse
)


class ProductsService:
    def __init__(self, client: Client):
        self.client = client

    def list_products(self, request: ListProductsRequest) -> ListProductsResponse:
        path = f"/portfolios/{request.portfolio_id}/products"
        query_params = append_pagination_params("", request.pagination)
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListProductsResponse(**response.json())

    def get_product_candles(self, request: GetProductCandlesRequest) -> GetProductCandlesResponse:
        path = f"/portfolios/{request.portfolio_id}/candles"
        query_params = f"product_id={request.product_id}&granularity={request.granularity}&start_time={request.start_time}&end_time={request.end_time}"
        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return GetProductCandlesResponse(**response.json())



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import List, Optional
from ...base_response import BaseResponse
from ...enums import OrderSide, OrderType, TimeInForce


@dataclass
class CreateOrderRequest:
    portfolio_id: str
    side: OrderSide
    client_order_id: str
    product_id: str
    type: OrderType
    base_quantity: Optional[str] = None
    quote_value: Optional[str] = None
    limit_price: Optional[str] = None
    start_time: Optional[str] = None
    expiry_time: Optional[str] = None
    time_in_force: Optional[TimeInForce] = None
    stp_id: Optional[str] = None
    display_quote_size: Optional[str] = None
    display_base_size: Optional[str] = None
    is_raise_exact: Optional[str] = None
    historical_pov: Optional[str] = None
    stop_price: Optional[str] = None
    settl_currency: Optional[str] = None
    post_only: Optional[bool] = None
    allowed_status_codes: Optional[List[int]] = None


@dataclass
class CreateOrderResponse(BaseResponse):
    order_id: str = None



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ...client import Client
from .list_portfolios import ListPortfoliosRequest, ListPortfoliosResponse
from .get_portfolio import GetPortfolioRequest, GetPortfolioResponse
from .get_counterparty_id import GetCounterpartyIdRequest, GetCounterpartyIdResponse


class PortfoliosService:
    def __init__(self, client: Client):
        self.client = client

    def list_portfolios(self, request: ListPortfoliosRequest) -> ListPortfoliosResponse:
        path = "/portfolios"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return ListPortfoliosResponse(**response.json())

    def get_portfolio(self, request: GetPortfolioRequest) -> GetPortfolioResponse:
        path = f"/portfolios/{request.portfolio_id}"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetPortfolioResponse(**response.json())

    def get_counterparty_id(self, request: GetCounterpartyIdRequest) -> GetCounterpartyIdResponse:
        path = f"/portfolios/{request.portfolio_id}/counterparty"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetCounterpartyIdResponse(**response.json())



# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from prime_sdk.client import Client
from prime_sdk.utils import to_body_dict
from .create_stake import (
    CreateStakeRequest,
    CreateStakeResponse
)
from .create_unstake import (
    CreateUnstakeRequest,
    CreateUnstakeResponse
)
from .create_portfolio_stake import (
    CreatePortfolioStakeRequest,
    CreatePortfolioStakeResponse
)
from .create_portfolio_unstake import (
    CreatePortfolioUnstakeRequest,
    CreatePortfolioUnstakeResponse
)


class StakingService:
    def __init__(self, client: Client):
        self.client = client

    def create_stake(self, request: CreateStakeRequest) -> CreateStakeResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/staking/initiate"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateStakeResponse(**response.json())

    def create_unstake(self, request: CreateUnstakeRequest) -> CreateUnstakeResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/staking/unstake"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreateUnstakeResponse(**response.json())

    def create_portfolio_stake(self, request: CreatePortfolioStakeRequest) -> CreatePortfolioStakeResponse:
        path = f"/portfolios/{request.portfolio_id}/staking/initiate"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreatePortfolioStakeResponse(**response.json())

    def create_portfolio_unstake(self, request: CreatePortfolioUnstakeRequest) -> CreatePortfolioUnstakeResponse:
        path = f"/portfolios/{request.portfolio_id}/staking/unstake"
        body = to_body_dict(request)
        response = self.client.request("POST", path, body=body, allowed_status_codes=request.allowed_status_codes)
        return CreatePortfolioUnstakeResponse(**response.json())



# Copyright 2024-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ...client import Client
from ...utils import append_query_param, append_pagination_params
from .get_wallet_balance import (
    GetWalletBalanceRequest,
    GetWalletBalanceResponse
)
from .list_entity_balances import (
    ListEntityBalancesRequest,
    ListEntityBalancesResponse
)
from .list_web3_wallet_balances import (
    ListWeb3WalletBalancesRequest,
    ListWeb3WalletBalancesResponse
)
from .list_portfolio_balances import (
    ListPortfolioBalancesRequest,
    ListPortfolioBalancesResponse
)


class BalancesService:
    def __init__(self, client: Client):
        self.client = client

    def get_wallet_balance(self, request: GetWalletBalanceRequest) -> GetWalletBalanceResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/balance"
        response = self.client.request("GET", path, allowed_status_codes=request.allowed_status_codes)
        return GetWalletBalanceResponse(**response.json())

    def list_entity_balances(self, request: ListEntityBalancesRequest) -> ListEntityBalancesResponse:
        path = f"/entities/{request.entity_id}/balances"

        query_params = append_query_param("", "symbols", request.symbols)
        query_params = append_query_param(query_params, "aggregation_type", request.aggregation_type)
        query_params = append_pagination_params(query_params, request.pagination)

        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListEntityBalancesResponse(**response.json())

    def list_web3_wallet_balances(self, request: ListWeb3WalletBalancesRequest) -> ListWeb3WalletBalancesResponse:
        path = f"/portfolios/{request.portfolio_id}/wallets/{request.wallet_id}/web3_balances"

        query_params = append_query_param("", 'visibility_statuses', request.visibility_statuses)
        query_params = append_pagination_params(query_params, request.pagination)

        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListWeb3WalletBalancesResponse(**response.json())

    def list_portfolio_balances(self, request: ListPortfolioBalancesRequest) -> ListPortfolioBalancesResponse:
        path = f"/portfolios/{request.portfolio_id}/balances"

        query_params = append_query_param("", 'symbols', request.symbols)
        query_params = append_query_param(query_params, 'balance_type', request.balance_type)
        query_params = append_pagination_params(query_params, request.pagination)

        response = self.client.request("GET", path, query=query_params, allowed_status_codes=request.allowed_status_codes)
        return ListPortfolioBalancesResponse(**response.json())


# Copyright 2025-present Coinbase Global, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Core classes
from .client import Client
from .credentials import Credentials

# Service classes - primary interface
from .services.activities import ActivitiesService
from .services.address_book import AddressBookService
from .services.allocations import AllocationsService
from .services.assets import AssetsService
from .services.balances import BalancesService
from .services.commission import CommissionService
from .services.financing import FinancingService
from .services.futures import FuturesService
from .services.invoices import InvoicesService
from .services.onchain_address_book import OnchainAddressBookService
from .services.orders import OrdersService
from .services.payment_methods import PaymentMethodsService
from .services.portfolios import PortfoliosService
from .services.positions import PositionsService
from .services.products import ProductsService
from .services.staking import StakingService
from .services.transactions import TransactionsService
from .services.users import UsersService
from .services.wallets import WalletsService

__all__ = [
    # Core classes
    "Client",
    "Credentials",
    
    # Service classes
    "ActivitiesService",
    "AddressBookService", 
    "AllocationsService",
    "AssetsService",
    "BalancesService",
    "CommissionService",
    "FinancingService",
    "FuturesService",
    "InvoicesService",
    "OnchainAddressBookService",
    "OrdersService",
    "PaymentMethodsService",
    "PortfoliosService",
    "PositionsService",
    "ProductsService",
    "StakingService",
    "TransactionsService",
    "UsersService",
    "WalletsService",
]

# For request/response classes, users should import directly from services:
# from prime_sdk.services.wallets import CreateWalletRequest, CreateWalletResponse
# from prime_sdk.services.transactions import CreateConversionRequest, CreateConversionResponse
# etc.